//////////////////////////////////////////
//                                      //
//      Minimap Plugin for DynRPG       //
//      Written by Kazesui              //
//      Version 1.00                    //
//                                      //
//////////////////////////////////////////

#define AUTO_DLLMAIN
#include <DynRPG/DynRPG.h>
#include <sstream>
#include <iomanip>
#include <fstream>

#include "Field.h"
#include "Target.h"

typedef std::map<std::string, Target*> trg;
std::map<std::string, Target*> targets;
RPG::Image* minimap = NULL;
RPG::Image* hero = NULL;
Field<int> tiles;

int height = 0;
int width = 0;

int srcx = 0;
int srcy = 0;
int srcxmax = 0;
int srcymax = 0;

int xnull = 234;
int ynull =   5;
int wsize =  40;
int hsize =  30;
int alpha = 230;

int fgcolor = 0xeb8c64;
int bgcolor = 0xebd764;

void loadMap()
{
    int width = RPG::map->getWidth();
    int height = RPG::map->getHeight();

    tiles.initField(width, height);

    std::stringstream ss;
    ss << std::setfill('0') << std::setw(4) << RPG::hero->mapId;
    std::string filename;
    filename = "Map" + ss.str() + ".lmu";

    std::ifstream mapfile(filename.c_str(), std::ios::binary);
    char buffer[3] = {0, 0,'\0'};
    while(buffer[0] != (char)0x47)mapfile.read(buffer,1);
    if(width*height < 0x2000)
        mapfile.read(buffer,2);
    else
        mapfile.read(buffer,3);

    unsigned char* chip = reinterpret_cast<unsigned char*>(RPG::map->chipset);
    for(int j = 0; j < height; j++)
    {
        for(int i = 0; i < width; i++)
        {
            mapfile.read(buffer, 2);
            int index = (unsigned char)buffer[1]<<8|(unsigned char)buffer[0];
            switch(index/1000)
            {
                case 0:
                    index = 0x154;
                    break;
                case 1:
                    index = 0x155;
                    break;
                case 2:
                    index = 0x156;
                    break;
                case 3:
                    index = (index - 0xb22)/0x32 + 0x154;
                    break;
                case 4:
                    index = (index - 0xe74)/0x32 + 0x154;
                    break;
                default:
                    index -= 0x1222;
            }

            tiles(i,j) = chip[index];
        }
    }
}

void drawMap()
{
    RPG::variables[10] = tiles(0,0);
    int p[256] = {0};
    p[1] = 0x0000ff;
    hero = RPG::Image::create(1,1);
    hero->setPalette(p);
    hero->applyPalette();
    hero->pixels[0] = 1;
    hero->alpha = alpha;

    width = RPG::map->getWidth();
    height = RPG::map->getHeight();
    srcxmax = (width  - wsize)<<1;
    srcymax = (height - hsize)<<1;

    p[1] = fgcolor;
    p[2] = bgcolor;
    minimap = RPG::Image::create(width*2,height*2);
    minimap->setPalette(p);
    minimap->applyPalette();
    minimap->alpha = alpha;
    int dex = 0;
    for(int i = 0; i < height; i += 1)
    {
        for(int j = 0; j < width; j += 1)
        {
            if(tiles(j,i)&32)
            {
                if(i > 0)
                {
                    if(tiles(j,i-1)&32)
                    {
                        minimap->pixels[dex+0] = 2;
                        minimap->pixels[dex+1] = 2;
                    }
                    else
                    {
                        minimap->pixels[dex+0] = 1;
                        minimap->pixels[dex+1] = 1;
                    }
                }
                else
                {
                    minimap->pixels[dex+0] = 2;
                    minimap->pixels[dex+1] = 2;
                }
            }
            else
            {
                minimap->pixels[dex+0] = ((~tiles(j,i)&8) || (~tiles(j,i)&2))?2:1;
                minimap->pixels[dex+1] = ((~tiles(j,i)&8) || (~tiles(j,i)&4))?2:1;
            }
            dex += 2;
        }
        for(int j = 0; j < width; j += 1)
        {
            if(tiles(j,i)&32)
            {
                if(i > 0)
                {
                    if(tiles(j,i-1)&32)
                    {
                        minimap->pixels[dex+0] = 2;
                        minimap->pixels[dex+1] = 2;
                    }
                    else
                    {
                        minimap->pixels[dex+0] = 1;
                        minimap->pixels[dex+1] = 1;
                    }
                }
                else
                {
                    minimap->pixels[dex+0] = 2;
                    minimap->pixels[dex+1] = 2;
                }
            }
            else
            {
                minimap->pixels[dex+0] = ((~tiles(j,i)&1) || (~tiles(j,i)&2))?2:1;
                minimap->pixels[dex+1] = ((~tiles(j,i)&1) || (~tiles(j,i)&4))?2:1;
            }
            dex += 2;
        }
    }
}

void onFrame(RPG::Scene scene)
{
    if(scene == RPG::SCENE_MAP && minimap)
    {
        int hero_x = ((RPG::hero->getScreenX()+RPG::map->getCameraX())>>3);
        int hero_y = ((RPG::hero->getScreenY()+RPG::map->getCameraY())>>3);

        srcx = hero_x - wsize;
        srcy = hero_y - hsize;
        if(srcx < 0) srcx = 0;
        if(srcy < 0) srcy = 0;
        if(srcx > srcxmax) srcx = srcxmax;
        if(srcy > srcymax) srcy = srcymax;

        hero_x += xnull - 1 - srcx;
        hero_y += ynull - 2 - srcy;

        RPG::screen->canvas->draw(xnull, ynull, minimap, srcx, srcy, 2*wsize, 2*hsize);

        trg::iterator itr = targets.begin();
        while(itr != targets.end())
        {
            itr->second->draw(srcx, srcy, xnull, ynull, wsize, hsize);
            ++itr;
        }
        RPG::screen->canvas->drawStretched(hero_x, hero_y, 2, 2, hero);
    }
}

int getColor(std::string col)
{
    if(!col.compare("red")) return COL_RED;
    if(!col.compare("blue")) return COL_BLUE;
    if(!col.compare("cyan")) return COL_CYAN;
    if(!col.compare("green")) return COL_GREEN;
    if(!col.compare("white")) return COL_WHITE;
    if(!col.compare("black")) return COL_BLACK;
    if(!col.compare("yellow")) return COL_YELLOW;
    if(!col.compare("magenta")) return COL_MAGENTA;
    return 0;
}

void onTitleScreen()
{
    if(minimap)return;
    RPG::Image::destroy(hero);
    RPG::Image::destroy(minimap);
    trg::iterator itr = targets.begin();
    while(itr != targets.end())
    {
        delete itr->second;
        itr++;
    }
    targets.clear();
    hero = NULL;
    minimap = NULL;
}

bool onComment( const char* text,
                const RPG::ParsedCommentData* 	parsedData,
                RPG::EventScriptLine* 	nextScriptLine,
                RPG::EventScriptData* 	scriptData,
                int 	eventId,
                int 	pageId,
                int 	lineId,
                int* 	nextLineId )
{
    std::string cmd = parsedData->command;
    if(!cmd.compare("make_target"))
    {
        trg::iterator itr = targets.find(parsedData->parameters[0].text);
        if(itr == targets.end())
        {
            targets[parsedData->parameters[0].text] = new Target();
        }
        return false;
    }
    if(!cmd.compare("set_hero_color"))
    {
        int p[256] = {0};
        if(parsedData->parameters[1].type == RPG::PARAM_NUMBER)
            p[1] = (int)parsedData->parameters[1].number|
                  ((int)parsedData->parameters[2].number<<8)|
                  ((int)parsedData->parameters[3].number<<16);
        else
            p[1] = getColor(parsedData->parameters[1].text);
        hero->setPalette(p);
        hero->applyPalette();
        return false;
    }

    if(!cmd.compare("set_target_color"))
    {
        trg::iterator itr = targets.find(parsedData->parameters[0].text);

        if(itr != targets.end())
        {
            if(parsedData->parameters[1].type == RPG::PARAM_NUMBER)
                itr->second->setColor((int)parsedData->parameters[1].number|
                                    ((int)parsedData->parameters[2].number<<8)|
                                    ((int)parsedData->parameters[3].number<<16));
            else
                itr->second->setColor(getColor(parsedData->parameters[1].text));
        }
        return false;
    }
    if(!cmd.compare("set_target_link"))
    {
        trg::iterator itr = targets.find(parsedData->parameters[0].text);
        if(itr != targets.end())
        {
            std::string type = parsedData->parameters[1].text;
            switch(parsedData->parameters[1].text[5])
            {
                case 'i':
                    itr->second->linkId(parsedData->parameters[2].number);
                    break;
                case 'v':
                    itr->second->linkVar(parsedData->parameters[2].number,parsedData->parameters[3].number);
                    break;
                case 'c':
                    itr->second->setCoordinates(parsedData->parameters[2].number,parsedData->parameters[3].number);
                    break;
                default:
                    MessageBox(NULL, "Invalid link command!", "Minimap Plugin", MB_ICONERROR);
            }
        }
        return false;
    }

    if(!cmd.compare("destroy_target"))
    {
        trg::iterator itr = targets.find(parsedData->parameters[0].text);
        if(itr != targets.end())
        {
            delete itr->second;
            targets.erase(itr);
        }
        return false;
    }

    if(!cmd.compare("set_alpha"))
    {
        alpha = parsedData->parameters[0].number;
        hero->alpha = alpha;
        minimap->alpha = alpha;

        trg::iterator itr = targets.begin();
        while(itr != targets.end())
        {
            itr->second->setAlpha(alpha);
            ++itr;
        }
        return false;
    }

    if(!cmd.compare("set_size"))
    {
        wsize = parsedData->parameters[0].number;
        hsize = parsedData->parameters[1].number;
        return false;
    }

    if(!cmd.compare("set_position"))
    {
        xnull = parsedData->parameters[0].number;
        ynull = parsedData->parameters[1].number;
        return false;
    }

    if(!cmd.compare("set_map_color"))
    {
        if(parsedData->parameters[0].type == RPG::PARAM_NUMBER)
        {
            fgcolor = (int)parsedData->parameters[0].number|
                     ((int)parsedData->parameters[1].number<<8)|
                     ((int)parsedData->parameters[2].number<<16);
            bgcolor = (int)parsedData->parameters[3].number|
                     ((int)parsedData->parameters[4].number<<8)|
                     ((int)parsedData->parameters[5].number<<16);
        }
        else
        {
            fgcolor = getColor(parsedData->parameters[0].text);
            bgcolor = getColor(parsedData->parameters[1].text);
        }
        int p[256] = {0};
        p[1] = fgcolor;
        p[2] = bgcolor;
        minimap->setPalette(p);
        minimap->applyPalette();
        return false;
    }

    if(!cmd.compare("make_minimap"))
    {
        if(!minimap)
        {
            loadMap();
            drawMap();
        }
        return false;
    }

    if(!cmd.compare("destroy_minimap"))
    {
        if(minimap)
        {
            RPG::Image::destroy(hero);
            RPG::Image::destroy(minimap);
            trg::iterator itr = targets.begin();
            while(itr != targets.end())
            {
                itr->second->destroy();
                delete itr->second;
                itr++;
            }
            targets.clear();
            hero = NULL;
            minimap = NULL;
        }
        return false;
    }

    if(!cmd.compare("is_minimap"))
    {
        RPG::switches[parsedData->parameters[0].number] = (bool)minimap;
        return false;
    }
    return true;
}
